@@ -66,6 +66,7 @@ def git(self, xml_parent, data):
     :arg str basedir: location relative to the workspace root to clone to
              (default: workspace)
     :arg bool skip-tag: Skip tagging
+    :arg bool shallow-clone: Perform shallow clone
     :arg bool prune: Prune remote branches
     :arg bool clean: Clean after checkout
     :arg bool fastpoll: Use fast remote polling
@@ -74,9 +75,15 @@ def git(self, xml_parent, data):
     :arg bool use-author: Use author rather than committer in Jenkin's build
       changeset
     :arg str git-tool: The name of the Git installation to use
+    :arg str reference-repo: Path of the reference repo to use during clone
+    :arg str scm-name: The unique scm name for this Git SCM
     :arg bool wipe-workspace: Wipe out workspace before build
     :arg str browser: what repository browser to use (default '(Auto)')
     :arg str browser-url: url for the repository browser
+    :arg str project-name: project name in Gitblit and ViewGit repobrowser
+    :arg str choosing-strategy: Jenkins class for selecting what to build
+    :arg str git-config-name: Configure name for Git clone
+    :arg str git-config-email: Configure email for Git clone
 
     :browser values:
         :githubweb:
@@ -89,6 +96,11 @@ def git(self, xml_parent, data):
         :redmineweb:
         :viewgit:
 
+    :choosing-strategy values:
+        :default:
+        :inverse:
+        :gerrit:
+
     Example::
 
       scm:
@@ -113,18 +125,24 @@ def git(self, xml_parent, data):
         ("wipe-workspace", 'wipeOutWorkspace', True),
         ("prune", 'pruneBranches', False),
         ("fastpoll", 'remotePoll', False),
-        (None, 'buildChooser', '', {
-            'class': 'hudson.plugins.git.util.DefaultBuildChooser'}),
         ("git-tool", 'gitTool', "Default"),
         (None, 'submoduleCfg', '', {'class': 'list'}),
         ('basedir', 'relativeTargetDir', ''),
-        (None, 'reference', ''),
-        (None, 'gitConfigName', ''),
-        (None, 'gitConfigEmail', ''),
+        ('reference-repo', 'reference', ''),
+        ("git-config-name", 'gitConfigName', ''),
+        ("git-config-email", 'gitConfigEmail', ''),
         ('skip-tag', 'skipTag', False),
-        (None, 'scmName', ''),
+        ('scm-name', 'scmName', ''),
+        ("shallow-clone", "useShallowClone", False),
     ]
 
+    choosing_strategies = {
+        'default': 'hudson.plugins.git.util.DefaultBuildChooser',
+        'gerrit': ('com.sonyericsson.hudson.plugins.'
+                   'gerrit.trigger.hudsontrigger.GerritTriggerBuildChooser'),
+        'inverse': 'hudson.plugins.git.util.InverseBuildChooser',
+    }
+
     scm = XML.SubElement(xml_parent,
                          'scm', {'class': 'hudson.plugins.git.GitSCM'})
     XML.SubElement(scm, 'configVersion').text = '2'
@@ -157,6 +175,15 @@ def git(self, xml_parent, data):
         urc = XML.SubElement(scm, 'userMergeOptions')
         XML.SubElement(urc, 'mergeRemote').text = name
         XML.SubElement(urc, 'mergeTarget').text = branch
+
+    try:
+        choosing_strategy = choosing_strategies[data.get('choosing-strategy',
+                                                         'default')]
+    except KeyError:
+        raise ValueError('Invalid choosing-strategy %r' %
+                         data.get('choosing-strategy'))
+    XML.SubElement(scm, 'buildChooser', {'class': choosing_strategy})
+
     for elem in mapping:
         (optname, xmlname, val) = elem[:3]
         attrs = {}
@@ -191,6 +218,92 @@ def git(self, xml_parent, data):
                             'hudson.plugins.git.browser.' +
                             browserdict[browser]})
         XML.SubElement(bc, 'url').text = data['browser-url']
+        if browser in ['gitblit', 'viewgit']:
+            XML.SubElement(bc, 'projectName').text = str(
+                data.get('project-name', ''))
+
+
+def repo(self, xml_parent, data):
+    """yaml: repo
+    Specifies the repo SCM repository for this job.
+    Requires the Jenkins `Repo Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Repo+Plugin>`_
+
+    :arg str manifest-url: URL of the repo manifest
+    :arg str manifest-branch: The branch of the manifest to use (optional)
+    :arg str manifest-file: Initial manifest file to use when initialising
+             (optional)
+    :arg str manifest-group: Only retrieve those projects in the manifest
+             tagged with the provided group name (optional)
+    :arg str destination-dir: Location relative to the workspace root to clone
+             under (optional)
+    :arg str repo-url: custom url to retrieve the repo application (optional)
+    :arg str mirror-dir: Path to mirror directory to reference when
+             initialising (optional)
+    :arg int jobs: Number of projects to fetch simultaneously (default 0)
+    :arg bool current-branch: Fetch only the current branch from the server
+              (default true)
+    :arg bool quiet: Make repo more quiet
+              (default true)
+    :arg str local-manifest: Contents of .repo/local_manifest.xml, written
+             prior to calling sync (optional)
+
+    Example::
+
+      scm:
+        - repo:
+            manifest-url: https://example.com/project/
+            manifest-branch: stable
+            manifest-file: repo.xml
+            manifest-group: drivers
+            destination-dir: build
+            repo-url: https://internal.net/projects/repo
+            mirror-dir: ~/git/project/
+            jobs: 3
+            current-branch: false
+            quiet: false
+            local-manifest: |
+              <?xml version="1.0" encoding="UTF-8"?>
+              <manifest>
+                <project path="external/project" name="org/project"
+                  remote="gerrit" revision="master" />
+              </manifest>
+    """
+
+    scm = XML.SubElement(xml_parent,
+                         'scm', {'class': 'hudson.plugins.repo.RepoScm'})
+
+    if 'manifest-url' in data:
+        XML.SubElement(scm, 'manifestRepositoryUrl').text = \
+            data['manifest-url']
+    else:
+        raise Exception("Must specify a manifest url")
+
+    mapping = [
+        # option, xml name, default value
+        ("manifest-branch", 'manifestBranch', ''),
+        ("manifest-file", 'manifestFile', ''),
+        ("manifest-group", 'manifestGroup', ''),
+        ("destination-dir", 'destinationDir', ''),
+        ("repo-url", 'repoUrl', ''),
+        ("mirror-dir", 'mirrorDir', ''),
+        ("jobs", 'jobs', 0),
+        ("current-branch", 'currentBranch', True),
+        ("quiet", 'quiet', True),
+        ("local-manifest", 'localManifest', ''),
+    ]
+
+    for elem in mapping:
+        (optname, xmlname, val) = elem
+        val = data.get(optname, val)
+        # Skip adding xml entry if default is empty string and no value given
+        if not val and elem[2] is '':
+            continue
+        xe = XML.SubElement(scm, xmlname)
+        if type(elem[2]) == bool:
+            xe.text = str(val).lower()
+        else:
+            xe.text = str(val)
 
 
 def svn(self, xml_parent, data):
@@ -309,9 +422,9 @@ def tfs(self, xml_parent, data):
 
         :web-access value:
             * **web-url** -- Enter the URL to the TSWA server. The plugin will
-            strip the last path (if any) of the URL when building URLs for
-            change set pages and other pages. (optional, default
-            uses server-url)
+              strip the last path (if any) of the URL when building URLs for
+              change set pages and other pages. (optional, default
+              uses server-url)
 
 
     Examples::
@@ -365,7 +478,7 @@ def tfs(self, xml_parent, data):
     XML.SubElement(tfs, 'userName').text = str(
         data.get('login', ''))
     XML.SubElement(tfs, 'useUpdate').text = str(
-        data.get('use-update', 'true'))
+        data.get('use-update', True))
     store = data.get('web-access', None)
     if 'web-access' in data and isinstance(store, list):
         web = XML.SubElement(tfs, 'repositoryBrowser', {'class': 'hudson.'
