@@ -140,7 +140,12 @@ def gerrit(parser, xml_parent, data):
     :arg int gerrit-build-successful-verified-value: Successful ''Verified''
         value
     :arg int gerrit-build-failed-verified-value: Failed ''Verified'' value
-    :arg str failure-message: Message to leave on failure
+    :arg int gerrit-build-successful-codereview-value: Successful
+        ''CodeReview'' value
+    :arg int gerrit-build-failed-codereview-value: Failed ''CodeReview'' value
+    :arg str failure-message: Message to leave on failure (default '')
+    :arg str successful-message: Message to leave on success (default '')
+    :arg str unstable-message: Message to leave when unstable (default '')
     :arg list projects: list of projects to match
 
       :Project: * **project-compare-type** (`str`) --  ''PLAIN'', ''ANT'' or
@@ -166,6 +171,22 @@ def gerrit(parser, xml_parent, data):
                   * **unstable** (`bool`)
                   * **notbuilt** (`bool`)
 
+    :arg bool silent:  When silent mode is on there will be no communication
+        back to Gerrit, i.e. no build started/failed/successful approve
+        messages etc. If other non-silent jobs are triggered by the same
+        Gerrit event as this job, the result of this job's build will not be
+        counted in the end result of the other jobs. (default false)
+    :arg bool escape-quotes: escape quotes in the values of gerrit change
+        parameters (default true)
+    :arg bool no-name-and-email: Do not pass compound 'name and email'
+        parameters (default false)
+    :arg bool dynamic-trigger-enabled: Enable/disable the dynamic trigger
+        (default false)
+    :arg str dynamic-trigger-url: if you specify this option, the gerrit
+        trigger configuration will be fetched from there on a regular interval
+    :arg str custom-url: Custom URL for a message sent to gerrit. Build
+        details URL will be used if empty. (default '')
+
     You may select one or more gerrit events upon which to trigger.
     You must also supply at least one project and branch, optionally
     more.  If you select the comment-added trigger, you should alse
@@ -196,6 +217,11 @@ def gerrit(parser, xml_parent, data):
                 failed: true
                 unstable: true
                 notbuilt: true
+            silent: false
+            escape-quotes: false
+            no-name-and-email: false
+            dynamic-trigger-enabled: true
+            dynamic-trigger-url: http://myhost/mytrigger
     """
 
     gerrit_handle_legacy_configuration(data)
@@ -231,19 +257,40 @@ def gerrit(parser, xml_parent, data):
                     file_path.get('compare-type', 'PLAIN')
                 XML.SubElement(fp_tag, 'pattern').text = file_path['pattern']
     build_gerrit_skip_votes(gtrig, data)
-    XML.SubElement(gtrig, 'silentMode').text = 'false'
-    XML.SubElement(gtrig, 'escapeQuotes').text = 'true'
+    XML.SubElement(gtrig, 'silentMode').text = str(
+        data.get('silent', False)).lower()
+    XML.SubElement(gtrig, 'escapeQuotes').text = str(
+        data.get('escape-quotes', True)).lower()
+    XML.SubElement(gtrig, 'noNameAndEmailParameters').text = str(
+        data.get('no-name-and-email', False)).lower()
+    XML.SubElement(gtrig, 'dynamicTriggerConfiguration').text = str(
+        data.get('dynamic-trigger-enabled', False))
+    XML.SubElement(gtrig, 'triggerConfigURL').text = str(
+        data.get('dynamic-trigger-url', ''))
     build_gerrit_triggers(gtrig, data)
-    if 'override-votes' in data and data['override-votes'] == 'true':
-        XML.SubElement(gtrig, 'gerritBuildSuccessfulVerifiedValue').text = \
-            str(data['gerrit-build-successful-verified-value'])
-        XML.SubElement(gtrig, 'gerritBuildFailedVerifiedValue').text = \
-            str(data['gerrit-build-failed-verified-value'])
-    XML.SubElement(gtrig, 'buildStartMessage')
-    XML.SubElement(gtrig, 'buildFailureMessage').text = data['failure-message']
-    XML.SubElement(gtrig, 'buildSuccessfulMessage')
-    XML.SubElement(gtrig, 'buildUnstableMessage')
-    XML.SubElement(gtrig, 'customUrl')
+    override = str(data.get('override-votes', False)).lower()
+    if override == 'true':
+        for yamlkey, xmlkey in [('gerrit-build-successful-verified-value',
+                                 'gerritBuildSuccessfulVerifiedValue'),
+                                ('gerrit-build-failed-verified-value',
+                                 'gerritBuildFailedVerifiedValue'),
+                                ('gerrit-build-successful-codereview-value',
+                                 'gerritBuildSuccesfulCodereviewValue'),
+                                ('gerrit-build-failed-codereview-value',
+                                 'gerritBuildFaiedCodeReviewValue')]:
+            if data.get(yamlkey) is not None:
+                # str(int(x)) makes input values like '+1' work
+                XML.SubElement(gtrig, xmlkey).text = str(
+                    int(data.get(yamlkey)))
+    XML.SubElement(gtrig, 'buildStartMessage').text = str(
+        data.get('start-message', ''))
+    XML.SubElement(gtrig, 'buildFailureMessage').text = \
+        data.get('failure-message', '')
+    XML.SubElement(gtrig, 'buildSuccessfulMessage').text = str(
+        data.get('successful-message', ''))
+    XML.SubElement(gtrig, 'buildUnstableMessage').text = str(
+        data.get('unstable-message', ''))
+    XML.SubElement(gtrig, 'customUrl').text = str(data.get('custom-url', ''))
 
 
 def pollscm(parser, xml_parent, data):
@@ -332,6 +379,73 @@ def github_pull_request(parser, xml_parent, data):
     XML.SubElement(ghprb, 'cron').text = data.get('cron', '')
 
 
+def build_result(parser, xml_parent, data):
+    """yaml: build-result
+    Configure jobB to monitor jobA build result. A build is scheduled if there
+    is a new build result matches your criteria (unstable, failure, ...)
+    Requires the Jenkins `BuildResultTrigger Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/BuildResultTrigger+Plugin>`_
+
+    :arg list groups: List groups of jobs and results to monitor for
+    :arg list jobs: The jobs to monitor (required)
+    :arg list results: Build results to monitor for (default success)
+    :arg bool combine: Combine all job information.  A build will be
+        scheduled only if all conditions are met (default false)
+    :arg str cron: The cron syntax with which to poll the jobs for the
+        supplied result (default '')
+
+    Example::
+
+      triggers:
+        - build-result:
+            combine: true
+            cron: '* * * * *'
+            groups:
+              - jobs:
+                  - foo
+                  - example
+                results:
+                  - unstable
+              - jobs:
+                  - foo2
+                results:
+                  - not-built
+                  - aborted
+    """
+    brt = XML.SubElement(xml_parent, 'org.jenkinsci.plugins.'
+                         'buildresulttrigger.BuildResultTrigger')
+    XML.SubElement(brt, 'spec').text = data.get('cron', '')
+    XML.SubElement(brt, 'combinedJobs').text = str(
+        data.get('combine', False)).lower()
+    jobs_info = XML.SubElement(brt, 'jobsInfo')
+    result_dict = {'success': 'SUCCESS',
+                   'unstable': 'UNSTABLE',
+                   'failure': 'FAILURE',
+                   'not-built': 'NOT_BUILT',
+                   'aborted': 'ABORTED'}
+    for group in data['groups']:
+        brti = XML.SubElement(jobs_info, 'org.jenkinsci.plugins.'
+                              'buildresulttrigger.model.'
+                              'BuildResultTriggerInfo')
+        if not group.get('jobs', []):
+            raise jenkins_jobs.errors.\
+                JenkinsJobsException('Jobs is missing and a required'
+                                     ' element')
+        jobs_string = ",".join(group['jobs'])
+        XML.SubElement(brti, 'jobNames').text = jobs_string
+        checked_results = XML.SubElement(brti, 'checkedResults')
+        for result in group.get('results', ['success']):
+            if result not in result_dict:
+                raise jenkins_jobs.errors.\
+                    JenkinsJobsException('Result entered is not valid,'
+                                         ' must be one of: '
+                                         + ', '.join(result_dict.keys()))
+            model_checked = XML.SubElement(checked_results, 'org.jenkinsci.'
+                                           'plugins.buildresulttrigger.model.'
+                                           'CheckedResult')
+            XML.SubElement(model_checked, 'checked').text = result_dict[result]
+
+
 class Triggers(jenkins_jobs.modules.base.Base):
     sequence = 50
 
